;;
;; Stitchart.scm
;;
;; This is the script extension for the GIMP.
;; Stit-ch-art creates a cross-stitching chart based on a given image.
;;
;; Copyright (c) 2006-2010, Stanislav Korotky, stasson@orc.ru
;;
;; This program is free software; you can redistribute it freely.
;; If you want to use the sources in derived products and/or modify
;; the script for your own purposes, you must keep the original
;; copyright statement.
;;
;; This program is distributed in the hope that it will be useful,
;; but WITHOUT ANY WARRANTY; without even the implied warranty of
;; MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
;;
;; Disclaimer
;; The script is not optimized anyhow. It was written in straight-
;; forward manner and in spite of the script-fu's shortcomings.
;; As the syntax is parser-friendly rather than human-friendly,
;; it implies that a human should not work on it a lot.


;; auxiliary functions

(define (floor x) (- x (fmod x 1)))


;; main script

(define (script-fu-stitchart input-img fake palette-name color-number auto-color-number font-name font-size auto-font-size stepx stepy linew)

; get source image
(let* (
       (old-img 0)
       (old-draw 0)
       (old-width 0)
       (old-height 0)
       (width 0)
       (height 0)
       (nx 0)
       (ny 0)
       (charmap 0)
       (extra 0)
       (extra-width 0)
       (new-img 0)
       (new-draw 0)
       (second-display 0)
       (x 0)
       (y 0)
       (linew-add 0)
       (color-array 0)
       (i 0)
       (j 0)
       (color-index 0)
       (color-pos 0)
       (column 0)
       (row 0)
       (color-name 0)
       (text-layer 0)
       (add-one 0)
       (logo-width 0)
       (logo-height 0)
      )

(set! old-img (car (gimp-image-duplicate input-img)))
(gimp-image-flatten old-img)
(set! old-draw (car (gimp-image-get-active-drawable old-img)))


; convert it to an indexed image with the palette

(if (= auto-color-number TRUE)

 (gimp-convert-indexed old-img 0 CUSTOM-PALETTE 256 0 0 palette-name)

 (begin
  (gimp-convert-indexed old-img 0 MAKE-PALETTE color-number 0 0 "")
  (gimp-image-convert-rgb old-img)
  (gimp-convert-indexed old-img 0 CUSTOM-PALETTE 256 0 0 palette-name) ;"Madeira256"
 )
)

; initialization

(if (= auto-font-size TRUE) (begin (set! font-size (+ (/ stepy 2) 2) )) )

(set! old-width (car (gimp-image-width old-img)))
(set! old-height (car (gimp-image-height old-img)))

(set! width (- (+ (* old-width stepx) stepx) 1))
(set! height (- (+ (* old-height stepy) stepy) 1))
; obsolet code, must be eliminated
(set! nx old-width)
(set! ny old-height)

; 50 symbols color map
(set! charmap
  '("#" "$" "%" "@" "*" "+" "=" "?" "<" ">"
    "0" "1" "2" "3" "4" "5" "6" "7" "8" "9"
    "A" "B" "C" "D" "E" "F" "G" "H" "J" "K"
    "L" "M" "N" "P" "R" "S" "T" "U" "V" "W"
    "X" "Y" "Z" "a" "b" "d" "e" "f" "g" "h"
   ))


; enlarge image for color legend

(set! extra (* (+ (floor (/ 256 ny)) 1) 250))
(set! extra-width (+ width extra))


; create new image for chart and fill it

(set! new-img (car (gimp-image-new extra-width height RGB)))
(set! new-draw (car (gimp-layer-new new-img extra-width height RGB-IMAGE "Blank" 100 NORMAL-MODE)))
(gimp-image-undo-disable new-img)
(gimp-image-add-layer new-img new-draw 0)
(gimp-palette-set-background '(255 255 255))
(gimp-edit-fill new-draw BACKGROUND-FILL)


; progress indicator

(set! second-display (car (gimp-display-new old-img)))
(gimp-progress-init "Please, wait..." second-display)


; draw pixel grid

(set! x 0)
(set! y 0)
(gimp-palette-set-foreground '(0 0 0))

(while (<= x nx)
  (set! linew-add 0)
  (if (= (fmod x 10) 0) (begin (set! linew-add (/ linew 2))))
  (gimp-rect-select new-img (+ (* x stepx) (/ stepx 2)) (/ stepy 2) (+ linew linew-add) (- height stepy) REPLACE 0 0)
  (gimp-edit-fill new-draw FG-IMAGE-FILL)
  (set! x (+ x 1)))

(while (<= y ny)
  (set! linew-add 0)
  (if (= (fmod y 10) 0) (begin (set! linew-add (/ linew 2))))
  (gimp-rect-select new-img (/ stepx 2) (+ (* y stepy) (/ stepy 2)) (- width stepx) (+ linew linew-add) REPLACE 0 0)
  (gimp-edit-fill new-draw FG-IMAGE-FILL)
  (set! y (+ y 1)))

(gimp-selection-none new-img)


; prepare color array for image analysis

(set! color-array (cons-array 256 'byte))


; cycle through the source image

(gimp-palette-set-foreground '(0 0 0))

(set! color-number 1)
(set! i 0)
(while (< i old-width)
  (set! j 0)
  (while (< j old-height)
    (set! color-index (aref (cadr (gimp-drawable-get-pixel old-draw i j)) 0))
    (if (< color-index 0) (begin (set! color-index (+ color-index 256) )))

    (if (= 0 (aref color-array color-index))
      (begin (aset color-array color-index color-number)
       (set! color-pos color-number)
       (set! color-number (+ color-number 1))
      )
      (set! color-pos (aref color-array color-index))
    )

    (set! color-pos (- color-pos 1))
    
    (if (< color-pos 50)
      (set! color-name (nth color-pos charmap))
      (set! color-name (number->string color-pos))
    )

    (set! text-layer (car (gimp-text-fontname new-img new-draw (+ (+ (* i stepx) (/ stepx 2)) (+ linew 2)) (+ (+ (* j stepy) (/ stepy 2)) (+ linew 1)) color-name 0 1 font-size 0 font-name))) ; "Arial"
    (gimp-floating-sel-anchor text-layer)
    (set! j (+ j 1))
  )
  (gimp-progress-update (/ i old-width))
  (set! i (+ i 1))
)


; now show color legend

(set! i 0)
(set! j 0)
(while (< i 256)
  (if (< 0 (aref color-array i))
    (begin ;output the used color to the legend
      (set! column (floor (/ j ny)))
      (set! row (fmod j ny))

      (set! color-pos (aref color-array i))
      (set! color-pos (- color-pos 1))
      (if (< color-pos 50)
        (set! color-name (nth color-pos charmap))
        (set! color-name (number->string color-pos))
      )

      (set! text-layer (car (gimp-text-fontname new-img new-draw (+ (+ width stepx) (* 250 column) ) (+ (+ (* row stepy) (/ stepy 2)) linew) (string-append color-name "	:" (car (gimp-palette-entry-get-name palette-name i))) 0 1 font-size 0 font-name))) ; "Madeira256", "Arial"
      (gimp-floating-sel-anchor text-layer)
      (set! j (+ j 1))
    ))
  (set! i (+ i 1))
)


; shrink the image if the color legend turned out to be smaller than planned
(set! add-one 0)
(if (< 0 (fmod j ny)) (begin (set! add-one 1) ) )
(set! extra (* (+ (floor (/ j ny)) add-one) 250))
(set! extra-width (+ width extra))
(gimp-layer-resize new-draw extra-width height 0 0)
(gimp-image-resize-to-layers new-img)


(set! logo-width (car (gimp-text-get-extents-fontname " Produced by Stit-ch-art for GIMP" 9 0 "Arial")))
(set! logo-height (cadr (gimp-text-get-extents-fontname " Produced by Stit-ch-art for GIMP" 9 0 "Arial")))
(set! text-layer (car (gimp-text-fontname new-img new-draw (- extra-width logo-width) (- height logo-height) " Produced by Stit-ch-art for GIMP" 0 0 9 0 "Arial")))
(gimp-drawable-transform-flip-simple text-layer ORIENTATION-HORIZONTAL 1 0 0)
(gimp-drawable-transform-flip-simple text-layer ORIENTATION-VERTICAL 1 0 0)
(gimp-floating-sel-anchor text-layer)


; finalization

(gimp-image-undo-enable new-img)
(gimp-display-new new-img)
(gimp-progress-update 1)
(gimp-display-delete second-display)
(gimp-displays-flush)

)
)


(script-fu-register "script-fu-stitchart"
  _"Stit-ch-art..."
  "Stit-ch-art creates a cross-stitching chart based on a given image. It applies the modified Madeira palette with 256 colors on the copy of the image, and then scans it pixel by pixel for color codes while generating black and white chart ready for printing (includes color legend from Madeira). Please note, that it may take a while for generating the chart for images of sizes 50*50 and larger. A handwork of something about 100*100 stitches should be considered as very large, and may require months of actual stitching. See readme.txt for details."
  "Stanislav Korotky"
  "Copyright (c) 2006-2010 Stanislav Korotky, All Rights Reserved"
  "2 May 2010"
  "RGB*"
  SF-IMAGE       "Input image"            0

  SF-DRAWABLE    "Input layer"            0

  SF-STRING      "Palette"                "Madeira256"
  SF-ADJUSTMENT  "Maximum Number of Colors" '(100 5 256 1 10 0 1)
  SF-TOGGLE      "Automatic Number of Colors"     TRUE
  SF-FONT        "Font"                   "Arial"
  SF-ADJUSTMENT  "Font Size"              '(10 5 50 1 5 0 1)
  SF-TOGGLE      "Automatic Font Size"    TRUE
  SF-ADJUSTMENT  "Pixels per Stitch Cell by X"   '(20 10 50 1 10 0 1)
  SF-ADJUSTMENT  "Pixels per Stitch Cell by Y"   '(20 10 50 1 10 0 1)
  SF-ADJUSTMENT  "Stitch Border Width (in pixels)" '(2  1   5 1  2 0 1)
)

(script-fu-menu-register "script-fu-stitchart"
			 _"<Image>/Script-Fu/Alchemy")
